/**
 * Tas.js
 * (c) 2017 Owen Luke
 * https://github.com/tasjs/tas
 * Released under the MIT License.
 */

var data = require('../../core/data');
var nested = require('./nested');
var indent = require('./indent');
var tree;

var forEach = {
	loopTimes: 0,

	pack: function(task){
		var func = task['init'];

		// on begin
		task['init'] = function(data, index){
			forEach.onBegin();
			func(data, index);
		};

		// on end
		// for printing a cut line at end of the forEach
		task['----'] = function(){
			forEach.onEnd();
		};

		task['----'].__ignore = true;
	},

	onBegin: function(){
		indent.backup();
		nested.add();

		data.extensions.isInForEach = true;
	},

	onEnd: function(){
		this.loopTimes = 0;

		indent.restore();
		indent.sub(); // the indent is increased 1 on begin, so here we should decrease 1.
		nested.sub();
		data.extensions.isInForEach = false;
	},

	setLoopTimes: function(val){
		this.loopTimes = val;
	},

	insertSerialNo: function(str){
		return data.extensions.isInForEach ? (str += ' ' + (this.loopTimes + 1) + ':') : str;
	},

	saveTree: function(tree_){
		tree = tree_;
	}
};

module.exports.__proto__ = forEach;
